import {
  csv,
  select,
  pie,
  arc,
  scaleOrdinal,
  format,
} from 'd3';

// Defining the colors for the pie chart sections.
const sectionColors = [
  'blue',
  'pink',
  'yellow',
  'lightgreen',
  'orange',
  'red',
  'brown',
  'lightblue',
  'violet',
  'lavender',
];

// Dataset URL
const csvUrl =
  'https://gist.githubusercontent.com/saikrish777/47ff342e130d4cc88e2b70480b8f4f99/raw/ecafed046a68a0dd70350254526c8da082078996/TestCricket_Records.csv';

// Function to fetch and parse the dataset
const fetchData = async () => {
  const data = await csv(csvUrl);
  // Aggregate data by Country, counting players
  const aggregation = {};
  data.forEach((d) => {
    if (aggregation[d.Country]) {
      aggregation[d.Country] += 1;
    } else {
      aggregation[d.Country] = 1;
    }
  });
  // Convert aggregation to array format suitable for D3 pie chart
  return Object.keys(aggregation).map((country) => ({
    Country: country,
    Players: aggregation[country],
  }));
};

// Select the SVG container
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

// Formatter for labels (e.g., converting large numbers)
const formatLabel = format('.2s');

// Function to render the pie chart
const render = (data) => {
  // Generate pie chart data
  const pieData = pie().value((d) => d.Players)(data);
  // Define color scale
  const colors = scaleOrdinal()
    .domain(data.map((d) => d.Country))
    .range(sectionColors);
  // Define arc generator
  const arcGenerator = arc()
    .innerRadius(0)
    .outerRadius(Math.min(width, height) / 2 - 10);
  // Positioning pie chart
  const pieGroup = svg
    .append('g')
    .attr(
      'transform',
      `translate(${width / 2},${height / 2})`,
    );
  // Draw pie sections
  pieGroup
    .selectAll('path')
    .data(pieData)
    .enter()
    .append('path')
    .attr('d', arcGenerator)
    .attr('fill', (d) => colors(d.data.Country));

  // Add labels to each section
  pieGroup
    .selectAll('text')
    .data(pieData)
    .enter()
    .append('text')
    .attr(
      'transform',
      (d) => `translate(${arcGenerator.centroid(d)})`,
    )
    .attr('text-anchor', 'middle')
    .text(
      (d) =>
        `${d.data.Country}: ${formatLabel(d.data.Players)}`,
    );

  // Construct legend
  // Construct legend with number of players
  const legend = svg
    .append('g')
    .attr('transform', 'translate(20,20)');
  data.forEach((d, i) => {
    const legendRow = legend
      .append('g')
      .attr('transform', `translate(0, ${i * 20})`);

    // Draw the color box
    legendRow
      .append('rect')
      .attr('width', 10)
      .attr('height', 10)
      .attr('fill', colors(d.Country));

    // Append country name
    legendRow
      .append('text')
      .attr('x', 15)
      .attr('y', 10)
      .text(d.Country);

    // Append number of players next to the country name
    legendRow
      .append('text')
      .attr('x', 150) // Adjust this value based on your SVG width and text length
      .attr('y', 10)
      .text(`(${d.Players})`); // Displaying the player count
  });
};

// Run the visualization
const run = async () => {
  const data = await fetchData();
  render(data);
};

run();
